﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MegaChip8EmulatorLib;
using System.Windows.Threading;
using System.Runtime.InteropServices;
using System.Media;

namespace MegaChip8Emulator
{
    /// <summary>
    /// Logique d'interaction pour MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        MegaChip8 Chip8Emulator = new MegaChip8();
        DispatcherTimer RenderTimer = new DispatcherTimer();
        WriteableBitmap writeableBitmap = new WriteableBitmap(64, 32, 60, 60, PixelFormats.Bgra32, null);

        DispatcherTimer Chip8Timer = new DispatcherTimer();

        public MainWindow()
        {
            InitializeComponent();
            frameBufferImage.Source = writeableBitmap;
        }

        private void MenuItem_Click(object sender, RoutedEventArgs e)
        {
            // On initialise un dialog d'ouverture de fichiers
            Microsoft.Win32.OpenFileDialog ofd = new Microsoft.Win32.OpenFileDialog();
            ofd.DefaultExt = ".ch8";
            ofd.Filter = "Chip8 ROM files (.ch8)|*.ch8";
            ofd.Multiselect = false;

            // On ouvre le dialog
            Nullable<bool> result = ofd.ShowDialog();

            // Si un fichier a été sélectionné
            if (result == true)
            {
                // On récupère son chemin
                string filename = ofd.FileName;

                // Et on l'ouvre
                using (System.IO.FileStream fs = new System.IO.FileStream(filename, System.IO.FileMode.Open))
                {
                    // On l'ouvre en mode binaire
                    using (System.IO.BinaryReader br = new System.IO.BinaryReader(fs))
                    {
                        // Et on le mets dans un tableau de byte
                        byte[] rom = new byte[fs.Length];
                        for (int i = 0; i < fs.Length; i++)
                            rom[i] = br.ReadByte();

                        // On stop l'émulation le temps du chargement
                        Chip8Timer.Stop();
                        // Puis on le donne à notre émulateur
                        Chip8Emulator.Load(rom);

                        RenderTimer.Interval = TimeSpan.FromSeconds(1 / 60);
                        RenderTimer.Tick += new EventHandler(Render);
                        RenderTimer.Start();

                        Chip8Timer.Interval = TimeSpan.FromSeconds(1 / 60);
                        Chip8Timer.Tick += new EventHandler(CPUCycle);
                        Chip8Timer.Start();
                    }
                }
            }
        }

        private void Render(object sender, EventArgs e)
        {
            byte[] data = Chip8Emulator.GetScreenBuffer();
            if (data != null)
            {
                // write our pixels into the bitmap source
                writeableBitmap.Lock();
                Marshal.Copy(data, 0, writeableBitmap.BackBuffer, data.Length);
                writeableBitmap.AddDirtyRect(
                    new Int32Rect(0, 0, 64, 32));
                writeableBitmap.Unlock();
            }
        }

        private void CPUCycle(object sender, EventArgs e)
        {
            Chip8Emulator.Emulate();
            if (Chip8Emulator._ST != 0)
                SystemSounds.Beep.Play();
        }

        private void MenuItem_Click_1(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void MenuItem_Click_2(object sender, RoutedEventArgs e)
        {
            Chip8Emulator.Reset();
        }

        private void MenuItem_Click_3(object sender, RoutedEventArgs e)
        {
            About about = new About();
            about.ShowDialog();
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.NumPad0)
                Chip8Emulator.Keyboard[0x0] = true;
            if (e.Key == Key.NumPad7)
                Chip8Emulator.Keyboard[0x1] = true;
            if (e.Key == Key.NumPad8)
                Chip8Emulator.Keyboard[0x2] = true;
            if (e.Key == Key.NumPad9)
                Chip8Emulator.Keyboard[0x3] = true;
            if (e.Key == Key.NumPad4)
                Chip8Emulator.Keyboard[0x4] = true;
            if (e.Key == Key.NumPad5)
                Chip8Emulator.Keyboard[0x5] = true;
            if (e.Key == Key.NumPad6)
                Chip8Emulator.Keyboard[0x6] = true;
            if (e.Key == Key.NumPad1)
                Chip8Emulator.Keyboard[0x7] = true;
            if (e.Key == Key.NumPad2)
                Chip8Emulator.Keyboard[0x8] = true;
            if (e.Key == Key.NumPad3)
                Chip8Emulator.Keyboard[0x9] = true;
            if (e.Key == Key.OemPlus)
                Chip8Emulator.Keyboard[0xA] = true;
            if (e.Key == Key.OemPeriod)
                Chip8Emulator.Keyboard[0xB] = true;
            if (e.Key == Key.Divide)
                Chip8Emulator.Keyboard[0xC] = true;
            if (e.Key == Key.Multiply)
                Chip8Emulator.Keyboard[0xD] = true;
            if (e.Key == Key.OemMinus)
                Chip8Emulator.Keyboard[0xE] = true;
            if (e.Key == Key.Enter)
                Chip8Emulator.Keyboard[0xF] = true;
        }

        private void Window_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.NumPad0)
                Chip8Emulator.Keyboard[0x0] = false;
            if (e.Key == Key.NumPad7)
                Chip8Emulator.Keyboard[0x1] = false;
            if (e.Key == Key.NumPad8)
                Chip8Emulator.Keyboard[0x2] = false;
            if (e.Key == Key.NumPad9)
                Chip8Emulator.Keyboard[0x3] = false;
            if (e.Key == Key.NumPad4)
                Chip8Emulator.Keyboard[0x4] = false;
            if (e.Key == Key.NumPad5)
                Chip8Emulator.Keyboard[0x5] = false;
            if (e.Key == Key.NumPad6)
                Chip8Emulator.Keyboard[0x6] = false;
            if (e.Key == Key.NumPad1)
                Chip8Emulator.Keyboard[0x7] = false;
            if (e.Key == Key.NumPad2)
                Chip8Emulator.Keyboard[0x8] = false;
            if (e.Key == Key.NumPad3)
                Chip8Emulator.Keyboard[0x9] = false;
            if (e.Key == Key.OemPlus)
                Chip8Emulator.Keyboard[0xA] = false;
            if (e.Key == Key.OemPeriod)
                Chip8Emulator.Keyboard[0xB] = false;
            if (e.Key == Key.Divide)
                Chip8Emulator.Keyboard[0xC] = false;
            if (e.Key == Key.Multiply)
                Chip8Emulator.Keyboard[0xD] = false;
            if (e.Key == Key.OemMinus)
                Chip8Emulator.Keyboard[0xE] = false;
            if (e.Key == Key.Enter)
                Chip8Emulator.Keyboard[0xF] = false;
        }
    }
}
